#include "StdAfx.h"
#include "PcmSelectFilter.h"

CPcmSelectFilter::CPcmSelectFilter(LPUNKNOWN pUnk, HRESULT *phr)
	: CTransformFilter(PCMSELFILTER_NAME, pUnk, CLSID_PCMSELFILTER)
	, m_iStereoMode(Pcm_Right)
{

}

CPcmSelectFilter::~CPcmSelectFilter(void)
{

}

CUnknown * WINAPI CPcmSelectFilter::CreateInstance(LPUNKNOWN pUnk, HRESULT *phr)
{
	// CX^X쐬
	CPcmSelectFilter *pNewFilter = new CPcmSelectFilter(pUnk, phr);
	if(!pNewFilter)*phr = E_OUTOFMEMORY;
	
	return dynamic_cast<CUnknown *>(pNewFilter);
}


HRESULT CPcmSelectFilter::CheckInputType(const CMediaType* mtIn)
{
    CheckPointer(mtIn, E_POINTER);

	if(*mtIn->Type() == MEDIATYPE_Audio){
		if(*mtIn->Subtype() == MEDIASUBTYPE_PCM){

				return S_OK;
			}
		}
	return VFW_E_TYPE_NOT_ACCEPTED;
}

HRESULT CPcmSelectFilter::CheckTransform(const CMediaType* mtIn, const CMediaType* mtOut)
{
    CheckPointer(mtIn, E_POINTER);
    CheckPointer(mtOut, E_POINTER);
    
	if(*mtOut->Type() == MEDIATYPE_Audio){
		if(*mtOut->Subtype() == MEDIASUBTYPE_PCM){			
			return S_OK;
			}
		}
    
	return VFW_E_TYPE_NOT_ACCEPTED;
}

HRESULT CPcmSelectFilter::CompleteConnect(PIN_DIRECTION direction,IPin *pReceivePin)
{
	return S_OK;
}

HRESULT CPcmSelectFilter::DecideBufferSize(IMemAllocator * pAllocator, ALLOCATOR_PROPERTIES *pprop)
{
	CAutoLock AutoLock(m_pLock);
	CheckPointer(pAllocator, E_POINTER);
	CheckPointer(pprop, E_POINTER);

	// obt@1΂悢
	if(!pprop->cBuffers)pprop->cBuffers = 1L;

	// Ƃ肠1MBm
	if(pprop->cbBuffer < 0x100000L)pprop->cbBuffer = 0x100000L;

	// AP[^vpeBݒ肵Ȃ
	ALLOCATOR_PROPERTIES Actual;
	HRESULT hr = pAllocator->SetProperties(pprop, &Actual);
	if(FAILED(hr))return hr;

	// v󂯓ꂽ
	if(Actual.cbBuffer < pprop->cbBuffer)return E_FAIL;

	return S_OK;
}


HRESULT CPcmSelectFilter::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	CAutoLock AutoLock(m_pLock);
	CheckPointer(pMediaType, E_POINTER);

	if(iPosition < 0)return E_INVALIDARG;
	if(iPosition > 0)return VFW_S_NO_MORE_ITEMS;
	*pMediaType=this->m_pInput->CurrentMediaType();
	return S_OK;
}

HRESULT CPcmSelectFilter::StartStreaming(void)
{
	return S_OK;
}

HRESULT CPcmSelectFilter::StopStreaming(void)
{
	return S_OK;
}


HRESULT CPcmSelectFilter::Transform(IMediaSample *pIn, IMediaSample *pOut)
{
	// ̓f[^|C^擾
	BYTE *pInData = NULL;
	pIn->GetPointer(&pInData);
	long lDataSize = pIn->GetActualDataLength();

	// o̓|C^擾
	pOut->SetActualDataLength(0);
	const DWORD dwOffset = pOut->GetActualDataLength();
	BYTE *pOutBuff = NULL;
	pOut->GetPointer(&pOutBuff);
	pOutBuff = &pOutBuff[dwOffset];

	// f[^𕡐
	DWORD dwSamples = lDataSize / 2 / 2;	// 16bit 2ch
	switch(m_iStereoMode){
	case Pcm_Left:
		// ̂		
		for(register DWORD dwPos = 0UL ; dwPos < dwSamples ; dwPos++){
				pOutBuff[dwPos * 2UL * 2UL + 0UL] = pInData[dwPos * 2UL * 2UL + 0UL];	// L
				pOutBuff[dwPos * 2UL * 2UL + 1UL] = pInData[dwPos * 2UL * 2UL + 1UL];	// L
				pOutBuff[dwPos * 2UL * 2UL + 2UL] = pInData[dwPos * 2UL * 2UL + 0UL];	// R
				pOutBuff[dwPos * 2UL * 2UL + 3UL] = pInData[dwPos * 2UL * 2UL + 1UL];	// R
			}
		break;
	case Pcm_Right:
		// Ê		
		for(register DWORD dwPos = 0UL ; dwPos < dwSamples ; dwPos++){
				pOutBuff[dwPos * 2UL * 2UL + 0UL] = pInData[dwPos * 2UL * 2UL + 2UL];	// L
				pOutBuff[dwPos * 2UL * 2UL + 1UL] = pInData[dwPos * 2UL * 2UL + 3UL];	// L
				pOutBuff[dwPos * 2UL * 2UL + 2UL] = pInData[dwPos * 2UL * 2UL + 2UL];	// R
				pOutBuff[dwPos * 2UL * 2UL + 3UL] = pInData[dwPos * 2UL * 2UL + 3UL];	// R
			}
		break;
	default:
		// ̂܂܏o
		::CopyMemory(pOutBuff,pInData,lDataSize);
		break;
		}

	pOut->SetActualDataLength(dwOffset + lDataSize);
	return S_OK;
}

bool CPcmSelectFilter::SetStereoMode(int iMode)
{
	switch(iMode){
	case 0:
	case 1:
	case 2:
		m_iStereoMode = iMode;
		break;
	default:
		return false;
		}
	return true;
}